@@ -31,6 +31,7 @@
 #include <QtGui/qpolygon.h>
 #include <QtGui/qtabbar.h>
 #include <QtGui/qtablewidget.h>
+#include <QtGui/qtextedit.h>
 #include <QtGui/qtextlayout.h>
 #include <QtGui/qtextobject.h>
 #include <QtGui/qtoolbar.h>
@@ -56,6 +57,13 @@
 #include <QtXml/qxmlstream.h>
 #endif
 
+#if QT_VERSION >= 0x040400
+#include <QtGui/qprinterinfo.h>
+#include <QtWebKit/qwebframe.h>
+#include <QtWebKit/qwebhistory.h>
+#include <QtNetwork/qnetworkcookie.h>
+#endif
+
 #include "smoke.h"
 
 #undef DEBUG
@@ -83,7 +91,7 @@ extern VALUE set_obj_info(const char * className, smokeruby_object * o);
 extern VALUE qt_internal_module;
 extern VALUE qvariant_class;
 extern bool application_terminated;
-};
+}
 
 extern bool isDerivedFromByName(Smoke *smoke, const char *className, const char *baseClassName);
 extern void mapPointer(VALUE obj, smokeruby_object *o, Smoke::Index classId, void *lastptr);
@@ -98,7 +106,7 @@ set_kde_resolve_classname(const char * (*kde_resolve_classname) (Smoke*, int, vo
 	_kde_resolve_classname = kde_resolve_classname;
 }
 
-};
+}
 
 void
 mark_qobject_children(QObject * qobject)
@@ -175,6 +183,15 @@ smokeruby_mark(void * p)
 	if (do_debug & qtdb_gc) qWarning("Checking for mark (%s*)%p", className, o->ptr);
 
     if (o->ptr && o->allocated) {
+		if (isDerivedFromByName(o->smoke, className, "QObject")) {
+			QObject * qobject = (QObject *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QObject"));
+			// Only mark the QObject tree if the current item doesn't have a parent.
+			// This avoids marking parts of a tree more than once.
+			if (qobject->parent() == 0) {
+				mark_qobject_children(qobject);
+			}
+		}
+
 		if (isDerivedFromByName(o->smoke, className, "QListWidget")) {
 			QListWidget * listwidget = (QListWidget *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QListWidget"));
 			
@@ -276,18 +293,9 @@ smokeruby_mark(void * p)
 
 		if (qstrcmp(className, "QModelIndex") == 0) {
 			QModelIndex * qmodelindex = (QModelIndex *) o->ptr;
-			obj = (VALUE) qmodelindex->internalPointer();
-			rb_gc_mark(obj);
-
-			return;
-		}
-
-		if (isDerivedFromByName(o->smoke, className, "QObject")) {
-			QObject * qobject = (QObject *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QObject"));
-			// Only mark the QObject tree if the current item doesn't have a parent.
-			// This avoids marking parts of a tree more than once.
-			if (qobject->parent() == 0) {
-				mark_qobject_children(qobject);
+			void * ptr = qmodelindex->internalPointer();
+			if (ptr != 0 && ptr != (void *) Qnil) {
+				rb_gc_mark((VALUE) ptr);
 			}
 
 			return;
@@ -1175,6 +1183,84 @@ void marshall_QByteArrayList(Marshall *m) {
     }
 }
 
+void marshall_ExtraSelectionList(Marshall *m) {
+	switch(m->action()) {
+	case Marshall::FromVALUE: 
+	{
+		VALUE list = *(m->var());
+		if (TYPE(list) != T_ARRAY) {
+			m->item().s_voidp = 0;
+			break;
+		}
+
+		int count = RARRAY(list)->len;
+		QList<QTextEdit::ExtraSelection> *selectionlist = new QList<QTextEdit::ExtraSelection>;
+
+		for (long i = 0; i < count; i++) {
+			VALUE item = rb_ary_entry(list, i);
+			QTextEdit::ExtraSelection selection;
+
+			VALUE cursor = rb_funcall(item, rb_intern("cursor"), 0);
+			smokeruby_object *c = value_obj_info(cursor);
+            selection.cursor = *((QTextCursor*) c->ptr);
+
+			VALUE format = rb_funcall(item, rb_intern("format"), 0);
+			smokeruby_object *f = value_obj_info(format);
+			selection.format = *((QTextCharFormat*) f->ptr);
+
+			selectionlist->append(selection);
+		}
+
+		m->item().s_voidp = selectionlist;
+		m->next();
+
+
+		if (m->cleanup()) {
+			delete selectionlist;
+		}
+		break;
+	}
+	case Marshall::ToVALUE: 
+	{
+		QList<QTextEdit::ExtraSelection> *selectionlist = static_cast<QList<QTextEdit::ExtraSelection>*>(m->item().s_voidp);
+		if (selectionlist == 0) {
+			*(m->var()) = Qnil;
+			break;
+		}
+
+	    VALUE av = rb_ary_new();
+		for (int i = 0; i < selectionlist->size(); i++) {
+			QTextEdit::ExtraSelection selection = selectionlist->at(i);
+
+			smokeruby_object  * c = alloc_smokeruby_object(	true, 
+															m->smoke(), 
+															m->smoke()->idClass("QTextCursor"), 
+															new QTextCursor(selection.cursor) );
+			VALUE cursor = set_obj_info("Qt::TextCursor", c);
+
+			smokeruby_object  * f = alloc_smokeruby_object(	true, 
+															m->smoke(), 
+															m->smoke()->idClass("QTextCharFormat"), 
+															new QTextCharFormat(selection.format) );
+			VALUE format = set_obj_info("Qt::TextCharFormat", f);
+
+			rb_ary_push(av, rb_funcall(qt_internal_module, rb_intern("create_extra_selection"), 2, cursor, format));
+		}
+
+
+		*(m->var()) = av;
+
+		if (m->cleanup()) {
+			delete selectionlist;
+		}
+	}
+	break;
+	default:
+		m->unsupported();
+		break;
+    }
+}
+
 template <class Item, class ItemList, const char *ItemSTR >
 void marshall_ItemList(Marshall *m) {
 	switch(m->action()) {
@@ -1268,6 +1354,57 @@ void marshall_ItemList(Marshall *m) {
    }
 }
 
+void marshall_QListCharStar(Marshall *m) {
+	switch(m->action()) {
+	case Marshall::FromVALUE:
+	{
+		VALUE av = *(m->var());
+		if (TYPE(av) != T_ARRAY) {
+			m->item().s_voidp = 0;
+			break;
+		}
+		int count = RARRAY(av)->len;
+		QList<const char*> *list = new QList<const char*>;
+		long i;
+		for(i = 0; i < count; i++) {
+			VALUE item = rb_ary_entry(av, i);
+			if (TYPE(item) != T_STRING) {
+				list->append(0);
+				continue;
+			}
+			list->append(StringValuePtr(item));
+		}
+
+		m->item().s_voidp = list;
+	}
+	break;
+	case Marshall::ToVALUE:
+	{
+		QList<const char*> *list = (QList<const char*>*)m->item().s_voidp;
+		if (list == 0) {
+			*(m->var()) = Qnil;
+			break;
+		}
+
+		VALUE av = rb_ary_new();
+
+		for (	QList<const char*>::iterator i = list->begin(); 
+				i != list->end(); 
+				++i ) 
+		{
+		    rb_ary_push(av, rb_str_new2((const char *)*i));
+		}
+		
+		*(m->var()) = av;
+		m->next();
+	}
+	break;
+      default:
+	m->unsupported();
+	break;
+    }
+}
+
 void marshall_QListInt(Marshall *m) {
     switch(m->action()) {
       case Marshall::FromVALUE:
@@ -1339,6 +1476,78 @@ void marshall_QListInt(Marshall *m) {
     }
 }
 
+
+void marshall_QListUInt(Marshall *m) {
+    switch(m->action()) {
+      case Marshall::FromVALUE:
+	{
+	    VALUE list = *(m->var());
+	    if (TYPE(list) != T_ARRAY) {
+		m->item().s_voidp = 0;
+		break;
+	    }
+	    int count = RARRAY(list)->len;
+	    QList<uint> *valuelist = new QList<uint>;
+	    long i;
+	    for(i = 0; i < count; i++) {
+		VALUE item = rb_ary_entry(list, i);
+		if(TYPE(item) != T_FIXNUM && TYPE(item) != T_BIGNUM) {
+		    valuelist->append(0);
+		    continue;
+		}
+		valuelist->append(NUM2UINT(item));
+	    }
+
+	    m->item().s_voidp = valuelist;
+	    m->next();
+
+		if (!m->type().isConst()) {
+			rb_ary_clear(list);
+	
+			for (	QList<uint>::iterator i = valuelist->begin(); 
+					i != valuelist->end(); 
+					++i ) 
+			{
+				rb_ary_push(list, UINT2NUM((int)*i));
+			}
+		}
+
+		if (m->cleanup()) {
+			delete valuelist;
+	    }
+	}
+	break;
+      case Marshall::ToVALUE:
+	{
+	    QList<uint> *valuelist = (QList<uint>*)m->item().s_voidp;
+	    if(!valuelist) {
+		*(m->var()) = Qnil;
+		break;
+	    }
+
+	    VALUE av = rb_ary_new();
+
+		for (	QList<uint>::iterator i = valuelist->begin(); 
+				i != valuelist->end(); 
+				++i ) 
+		{
+		    rb_ary_push(av, UINT2NUM((int)*i));
+		}
+		
+	    *(m->var()) = av;
+		m->next();
+
+		if (m->cleanup()) {
+			delete valuelist;
+		}
+	}
+	break;
+      default:
+	m->unsupported();
+	break;
+    }
+}
+
 void marshall_QListqreal(Marshall *m) {
     switch(m->action()) {
       case Marshall::FromVALUE:
@@ -2095,7 +2304,7 @@ void marshall_QPairintint(Marshall *m) {
     }
 }
 
-#define DEF_LIST_MARSHALLER(ListIdent,ItemList,Item) namespace { char ListIdent##STR[] = #Item; };  \
+#define DEF_LIST_MARSHALLER(ListIdent,ItemList,Item) namespace { char ListIdent##STR[] = #Item; }  \
         Marshall::HandlerFn marshall_##ListIdent = marshall_ItemList<Item,ItemList,ListIdent##STR>;
 
 DEF_LIST_MARSHALLER( QAbstractButtonList, QList<QAbstractButton*>, QAbstractButton )
@@ -2121,6 +2330,10 @@ DEF_LIST_MARSHALLER( QUndoStackList, QList<QUndoStack*>, QUndoStack )
 DEF_LIST_MARSHALLER( QMdiSubWindowList, QList<QMdiSubWindow*>, QMdiSubWindow )
 #endif
 
+#if QT_VERSION >= 0x40400
+DEF_LIST_MARSHALLER( QWebFrameList, QList<QWebFrame*>, QWebFrame )
+#endif
+
 template <class Item, class ItemList, const char *ItemSTR >
 void marshall_ValueListItem(Marshall *m) {
 	switch(m->action()) {
@@ -2230,7 +2443,7 @@ void marshall_ValueListItem(Marshall *m) {
 	}
 }
 
-#define DEF_VALUELIST_MARSHALLER(ListIdent,ItemList,Item) namespace { char ListIdent##STR[] = #Item; };  \
+#define DEF_VALUELIST_MARSHALLER(ListIdent,ItemList,Item) namespace { char ListIdent##STR[] = #Item; }  \
         Marshall::HandlerFn marshall_##ListIdent = marshall_ValueListItem<Item,ItemList,ListIdent##STR>;
 
 DEF_VALUELIST_MARSHALLER( QColorVector, QVector<QColor>, QColor )
@@ -2269,6 +2482,12 @@ DEF_VALUELIST_MARSHALLER( QXmlStreamNamespaceDeclarations, QVector<QXmlStreamNam
 DEF_VALUELIST_MARSHALLER( QXmlStreamNotationDeclarations, QVector<QXmlStreamNotationDeclaration>, QXmlStreamNotationDeclaration )
 #endif
 
+#if QT_VERSION >= 0x40400
+DEF_VALUELIST_MARSHALLER( QNetworkCookieList, QList<QNetworkCookie>, QNetworkCookie )
+DEF_VALUELIST_MARSHALLER( QPrinterInfoList, QList<QPrinterInfo>, QPrinterInfo )
+DEF_VALUELIST_MARSHALLER( QWebHistoryItemList, QList<QWebHistoryItem>, QWebHistoryItem )
+#endif
+
 TypeHandler Qt_handlers[] = {
     { "bool*", marshall_it<bool *> },
     { "bool&", marshall_it<bool *> },
@@ -2287,11 +2506,17 @@ TypeHandler Qt_handlers[] = {
     { "QFileInfoList", marshall_QFileInfoList },
     { "QGradiantStops", marshall_QPairqrealQColor },
     { "QGradiantStops&", marshall_QPairqrealQColor },
+    { "unsigned int&", marshall_it<unsigned int *> },
+    { "quint32&", marshall_it<unsigned int *> },
+    { "uint&", marshall_it<unsigned int *> },
     { "qint32&", marshall_it<int *> },
     { "qint64", marshall_it<long long> },
     { "qint64&", marshall_it<long long> },
+    { "QList<const char*>", marshall_QListCharStar },
     { "QList<int>", marshall_QListInt },
     { "QList<int>&", marshall_QListInt },
+    { "QList<uint>", marshall_QListUInt },
+    { "QList<uint>&", marshall_QListUInt },
     { "QList<QAbstractButton*>", marshall_QAbstractButtonList },
     { "QList<QActionGroup*>", marshall_QActionGroupList },
     { "QList<QAction*>", marshall_QActionList },
@@ -2317,12 +2542,16 @@ TypeHandler Qt_handlers[] = {
     { "QList<QRectF>", marshall_QRectFList },
     { "QList<QRectF>&", marshall_QRectFList },
     { "QList<qreal>", marshall_QListqreal },
+    { "QList<double>", marshall_QListqreal },
+    { "QList<double>&", marshall_QListqreal },
     { "QList<QStandardItem*>", marshall_QStandardItemList },
     { "QList<QStandardItem*>&", marshall_QStandardItemList },
     { "QList<QTableWidgetItem*>", marshall_QTableWidgetItemList },
     { "QList<QTableWidgetItem*>&", marshall_QTableWidgetItemList },
     { "QList<QTableWidgetSelectionRange>", marshall_QTableWidgetSelectionRangeList },
     { "QList<QTextBlock>", marshall_QTextBlockList },
+    { "QList<QTextEdit::ExtraSelection>", marshall_ExtraSelectionList },
+    { "QList<QTextEdit::ExtraSelection>&", marshall_ExtraSelectionList },
     { "QList<QTextFrame*>", marshall_QTextFrameList },
     { "QList<QTextLayout::FormatRange>", marshall_QTextLayoutFormatRangeList },
     { "QList<QTextLayout::FormatRange>&", marshall_QTextLayoutFormatRangeList },
@@ -2418,10 +2647,18 @@ TypeHandler Qt_handlers[] = {
     { "QList<QSslCertificate>&", marshall_QSslCertificateList },
     { "QList<QSslCipher>", marshall_QSslCipherList },
     { "QList<QSslCipher>&", marshall_QSslCipherList },
+    { "QList<QSslError>", marshall_QSslErrorList },
     { "QList<QSslError>&", marshall_QSslErrorList },
     { "QXmlStreamEntityDeclarations", marshall_QXmlStreamEntityDeclarations },
     { "QXmlStreamNamespaceDeclarations", marshall_QXmlStreamNamespaceDeclarations },
     { "QXmlStreamNotationDeclarations", marshall_QXmlStreamNotationDeclarations },
+#endif
+#if QT_VERSION >= 0x040400
+    { "QList<QNetworkCookie>", marshall_QNetworkCookieList },
+    { "QList<QNetworkCookie>&", marshall_QNetworkCookieList },
+    { "QList<QPrinterInfo>", marshall_QPrinterInfoList },
+    { "QList<QWebFrame*>", marshall_QWebFrameList },
+    { "QList<QWebHistoryItem>", marshall_QWebHistoryItemList },
 #endif
     { 0, 0 }
 };
