@@ -195,6 +195,16 @@ module Qt
 		end
 	end # Qt::Base
 
+	class AbstractSlider < Qt::Base
+		def range=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
+
 	class AbstractSocket < Qt::Base
 		def abort(*args)
 			method_missing(:abort, *args)
@@ -400,11 +410,26 @@ module Qt
 			if args.size == 1 && args[0].class.name == "DateTime"
 				return super(	Qt::Date.new(args[0].year, args[0].month, args[0].day), 
 								Qt::Time.new(args[0].hour, args[0].min, args[0].sec) )
+			elsif args.size == 1 && args[0].class.name == "Time"
+				result = super(	Qt::Date.new(args[0].year, args[0].month, args[0].day), 
+								Qt::Time.new(args[0].hour, args[0].min, args[0].sec, args[0].usec / 1000) )
+				result.timeSpec = (args[0].utc? ? Qt::UTC : Qt::LocalTime)
+				return result
 			else
 				return super(*args)
 			end
 		end
 
+		def to_time
+			if timeSpec == Qt::UTC
+				return ::Time.utc(	date.year, date.month, date.day,
+									time.hour, time.minute, time.second, time.msec * 1000 )
+			else
+				return ::Time.local(	date.year, date.month, date.day,
+										time.hour, time.minute, time.second, time.msec * 1000 )
+			end
+		end
+
 		def inspect
 			str = super
 			str.sub(/>$/, " %s>" % toString)
@@ -416,6 +441,18 @@ module Qt
 		end
 	end
 
+	class DBusArgument < Qt::Base
+		def inspect
+			str = super
+			str.sub(/>$/, " currentSignature='%s', atEnd=%s>" % [currentSignature, atEnd])
+		end
+		
+		def pretty_print(pp)
+			str = to_s
+			pp.text str.sub(/>$/, " currentSignature='%s, atEnd=%s'>" % [currentSignature, atEnd])
+		end
+	end
+
 	class DBusConnection < Qt::Base
 		def send(*args)
 			method_missing(:send, *args)
@@ -494,7 +531,13 @@ module Qt
 		def value
 			if type() == Qt::DBusMessage::ReplyMessage
 				reply = arguments()
-				return reply.length > 0 ? reply[0].value : nil
+				if reply.length == 0
+					return nil
+				elsif reply.length == 1
+					return reply[0].value
+				else
+					return reply.collect {|v| v.value}
+				end
 			else
 				return nil
 			end
@@ -546,6 +589,16 @@ module Qt
 		end
 	end
 
+	class Dial < Qt::Base
+		def range=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
+
 	class Dialog < Qt::Base
 		def exec(*args)
 			method_missing(:exec, *args)
@@ -558,6 +611,26 @@ module Qt
 		end
 	end
 
+	class DoubleSpinBox < Qt::Base
+		def range=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
+
+	class DoubleValidator < Qt::Base
+		def range=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
+
 	class DomDocumentType < Qt::Base
 		def name(*args)
 			method_missing(:name, *args)
@@ -863,6 +936,16 @@ module Qt
 		end
 	end
 
+	class IntValidator < Qt::Base
+		def range=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
+
 	class ItemSelection < Qt::Base
 		def select(*args)
 			method_missing(:select, *args)
@@ -879,6 +962,12 @@ module Qt
 		end
 	end
 	
+	class KeyEvent < Qt::Base 
+		def type(*args)
+			method_missing(:type, *args)
+		end
+	end
+
 	class KeySequence < Qt::Base
 		def initialize(*args)
 			if args.length == 1 && args[0].kind_of?(Qt::Enum) && args[0].type == "Qt::Key"
@@ -1197,12 +1286,54 @@ module Qt
 		end
 	end
 
+	class Polygon < Qt::Base
+		include Enumerable
+
+		def each
+			for i in 0...count
+				yield point(i)
+			end
+			return self
+		end
+	end
+
+	class PolygonF < Qt::Base
+		include Enumerable
+
+		def each
+			for i in 0...count
+				yield point(i)
+			end
+			return self
+		end
+	end
+
 	class PrintDialog < Qt::Base
 		def exec(*args)
 			method_missing(:exec, *args)
 		end
 	end
 
+	class ProgressBar < Qt::Base
+		def range=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
+
+	class ProgressDialog < Qt::Base
+		def range=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
+
 	class Printer < Qt::Base
 		def abort(*args)
 			method_missing(:abort, *args)
@@ -1309,6 +1440,16 @@ module Qt
 		end
 	end
 
+	class ScrollBar < Qt::Base
+		def range=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
+
 	class Shortcut < Qt::Base
 		def id(*args)
 			method_missing(:id, *args)
@@ -1363,12 +1504,32 @@ module Qt
 		end
 	end
 
+	class Slider < Qt::Base
+		def range=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
+
 	class SocketNotifier < Qt::Base
 		def type(*args)
 			method_missing(:type, *args)
 		end
 	end
 
+	class SpinBox < Qt::Base
+		def range=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
+
 	class SqlDatabase < Qt::Base
 		def exec(*args)
 			method_missing(:exec, *args)
@@ -1497,6 +1658,10 @@ module Qt
 		end
 	end
 
+	class TextEdit < Qt::Base
+		ExtraSelection = Struct.new(:cursor, :format)
+	end
+
 	class TextFormat < Qt::Base
 		def type(*args)
 			method_missing(:type, *args)
@@ -1570,6 +1735,16 @@ module Qt
 			method_missing(:type, *args)
 		end
 	end
+
+	class TimeLine < Qt::Base
+		def frameRange=(arg)
+			if arg.kind_of? Range
+				return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+			else
+				return super(arg)
+			end
+		end
+	end
 	
 	class ToolButton < Qt::Base 
 		def setShortcut(arg)
@@ -1663,6 +1838,18 @@ module Qt
 		end
 	end
 
+	class Url < Qt::Base
+		def inspect
+			str = super
+			str.sub(/>$/, " url=%s>" % toString)
+		end
+		
+		def pretty_print(pp)
+			str = to_s
+			pp.text str.sub(/>$/, " url=%s>" % toString)
+		end
+	end
+
 	class UrlInfo < Qt::Base
 		def name(*args)
 			method_missing(:name, *args)
@@ -1798,6 +1985,10 @@ module Qt
 				return qVariantValue(Qt::DBusArgument, self)
             when "QDBusVariant"
 				return qVariantValue(Qt::Variant, self)
+            when "QDBusObjectPath"
+				return qdbusobjectpath_value(self)
+            when "QDBusSignature"
+				return qdbussignature_value(self)
             end
 		end
 
@@ -1942,10 +2133,9 @@ module Qt
 	# method resolution when two methods differ only by an enum type.
 	class Enum
 		attr_accessor :type, :value
-		def initialize(n, type)
-			super() 
+		def initialize(n, enum_type)
 			@value = n 
-			@type = type
+			@type = enum_type
 		end
 		
 		def +(n) 
@@ -2062,26 +2252,33 @@ module Qt
 		@@idclass   = []
 
 		def Internal.normalize_classname(classname)
-			if classname =~ /^Qext/
-				now = classname.sub(/^Qext(?=[A-Z])/,'Qext::')
+			if classname =~ /^Qsci/
+				now = classname.sub(/^Qsci(?=[A-Z])/,'Qsci::')
 			elsif classname =~ /^Qwt/
 				now = classname.sub(/^Qwt(?=[A-Z])/,'Qwt::')
 			elsif classname =~ /^Q3/
 				now = classname.sub(/^Q3(?=[A-Z])/,'Qt3::')
 			elsif classname =~ /^Q/
 				now = classname.sub(/^Q(?=[A-Z])/,'Qt::')
-			elsif classname =~ /^(KConfigSkeleton|KWin)::/
+			elsif classname =~ /^KCoreConfigSkeleton::/
+				# Make classes under KCoreConfigSkeleton appear under KDE::ConfigSkeleton
+				# in Ruby, as the KCoreConfigSkeleton class isn't really the public api
+				now = classname.sub(/KCore/,'KDE::')
+			elsif classname =~ /^(KConfigSkeleton|KWin|KDateTime|KTimeZone)::/
 				now = classname.sub(/^K?(?=[A-Z])/,'KDE::')
 			elsif classname !~ /::/
 				now = classname.sub(/^K?(?=[A-Z])/,'KDE::')
 			else
 				now = classname
 			end
-#			puts "normalize_classname = was::#{classname}, now::#{now}"
+#			puts "normalize_classname = was: #{classname}, now: #{now}"
 			now
 		end
 
 		def Internal.init_class(c)
+			if c == "WebCore" || c == "std"
+				return
+			end
 			classname = Qt::Internal::normalize_classname(c)
 			classId = Qt::Internal.idClass(c)
 			insert_pclassid(classname, classId)
@@ -2099,8 +2296,10 @@ module Qt
 		def Internal.checkarg(argtype, typename)
 			puts "      #{typename} (#{argtype})" if debug_level >= DebugLevel::High
 			if argtype == 'i'
-				if typename =~ /^int&?$|^signed int&?$|^signed$|^qint32&?$|^quint32&?$/
+				if typename =~ /^int&?$|^signed int&?$|^signed$|^qint32&?$/
 					return 2
+				elsif typename =~ /^quint32&?$/
+					return 1
 				elsif typename =~ /^(?:short|ushort|unsigned short int|uchar|uint|long|ulong|unsigned long int|unsigned|float|double|WId|Q_PID|^quint16&?$|^qint16&?$)$/
 					return 1
 				elsif typename =~ /^(quint|qint|qulong|qlong|qreal)/
@@ -2172,6 +2371,7 @@ module Qt
 				end
 			else
 				t = typename.sub(/^const\s+/, '')
+				t.sub!(/(::)?Ptr$/, '')
 				t.sub!(/[&*]$/, '')
 				if argtype == t
 					return 1
@@ -2186,7 +2386,6 @@ module Qt
 		end
 
 		def Internal.find_class(classname)
-			# puts @@classes.keys.sort.join "\n"
 			@@classes[classname]
 		end
 		
@@ -2230,7 +2429,8 @@ module Qt
 			
 			if method == "new"
 				method = classname.dup 
-				method.gsub!(/^(QTextLayout|KParts|KIO|KNS|DOM|Kontact|Kate|KTextEditor|KConfigSkeleton::ItemEnum|KConfigSkeleton|KWin)::/,"")
+#				method.gsub!(/^(QTextBlock|QTextLayout|QTextFrame|QTextOption|QLocale|QAbstractFileEngine|KDateTime|KBookmark|KParts|KMediaPlayer|KIO|KNS|DOM|Kontact|Kate|KSettings|KTimeZone|KTextEditor|KWin|KWallet|Plasma|Sonnet|Soprano|Nepomuk)::/,"")
+				method.gsub!(/^.*::/,"")
 			end
 			method = "operator" + method.sub("@","") if method !~ /[a-zA-Z]+/
 			# Change foobar= to setFoobar()					
@@ -2261,7 +2461,7 @@ module Qt
 			methodIds = []
 			methods.collect { |meth| methodIds.concat( findMethod(classname, meth) ) }
 			
-			if method =~ /_/ && methodIds.length == 0
+			if method =~ /._./ && methodIds.length == 0
 				# If the method name contains underscores, convert to camel case
 				# form and try again
 				method.gsub!(/(.)_(.)/) {$1 + $2.upcase}
@@ -2351,8 +2551,8 @@ module Qt
 			return num.value = val
 		end
 		
-		def Internal.create_qenum(num, type)
-			return Qt::Enum.new(num, type)
+		def Internal.create_qenum(num, enum_type)
+			return Qt::Enum.new(num, enum_type)
 		end
 		
 		def Internal.get_qenum_type(e)
@@ -2367,6 +2567,10 @@ module Qt
 			return b.value = val
 		end
 
+		def Internal.create_extra_selection(cursor, format)
+			Qt::TextEdit::ExtraSelection.new(cursor, format)
+		end
+
 		def Internal.getAllParents(class_id, res)
 			getIsa(class_id).each do |s|
 				c = idClass(s)
@@ -2374,31 +2578,6 @@ module Qt
 				getAllParents(c, res)
 			end
 		end
-	
-		def Internal.signalInfo(qobject, signal_name)
-			signals = Meta[qobject.class.name].get_signals
-			signals.each_with_index do |signal, i|
-				if signal.name == signal_name
-					return [signal.reply_type, signal.full_name, i]
-				end
-			end
-		end
-	
-		def Internal.getMocArguments(reply_type, member)
-			argStr = member.sub(/.*\(/, '').sub(/\)$/, '')
-			args = argStr.scan(/([^,]*<[^>]+>)|([^,]+)/)
-			args.unshift reply_type
-			mocargs = allocateMocArguments(args.length)
-			args.each_with_index do |arg, i|
-				arg = arg.to_s
-				a = arg.sub(/^const\s+/, '')
-				a = (a =~ /^(bool|int|double|char\*|QString)&?$/) ? $1 : 'ptr'
-				valid = setMocType(mocargs, i, arg, a)
-			end
-			result = []
-			result << args.length << mocargs
-			result
-		end
 
 		#
 		# From the enum MethodFlags in qt-copy/src/tools/moc/generator.cpp
@@ -2531,6 +2710,12 @@ module Qt
 										Qt::SignalBlockInvocation.new(src, block, signature),
 										SLOT(signature) )
 		end
+
+		def Internal.single_shot_timer_connect(interval, target, block)
+			return Qt::Timer.singleShot(	interval,
+											Qt::BlockInvocation.new(target, block, "invoke()"),
+											SLOT("invoke()") )
+		end
 	end # Qt::Internal
 
 	Meta = {}
